@@ -38,6 +38,24 @@
 namespace lightspark
 {
 
+class u8
+{
+friend std::istream& operator>>(std::istream& in, u8& v);
+private:
+	uint32_t val;
+public:
+	operator uint32_t() const{return val;}
+};
+
+class u16
+{
+friend std::istream& operator>>(std::istream& in, u16& v);
+private:
+	uint32_t val;
+public:
+	operator uint32_t() const{return val;}
+};
+
 class s24
 {
 friend std::istream& operator>>(std::istream& in, s24& v);
@@ -47,6 +65,17 @@ public:
 	operator int32_t(){return val;}
 };
 
+class u30
+{
+friend std::istream& operator>>(std::istream& in, u30& v);
+private:
+	uint32_t val;
+public:
+	u30():val(0){}
+	u30(int v):val(v){}
+	operator uint32_t() const{return val;}
+};
+
 class s32
 {
 friend std::istream& operator>>(std::istream& in, s32& v);
@@ -65,35 +94,6 @@ public:
 	operator uint32_t() const{return val;}
 };
 
-class u30
-{
-friend std::istream& operator>>(std::istream& in, u30& v);
-private:
-	uint32_t val;
-public:
-	u30():val(0){}
-	u30(int v):val(v){}
-	operator uint32_t() const{return val;}
-};
-
-class u16
-{
-friend std::istream& operator>>(std::istream& in, u16& v);
-private:
-	uint32_t val;
-public:
-	operator uint32_t() const{return val;}
-};
-
-class u8
-{
-friend std::istream& operator>>(std::istream& in, u8& v);
-private:
-	uint32_t val;
-public:
-	operator uint32_t() const{return val;}
-};
-
 class d64
 {
 friend std::istream& operator>>(std::istream& in, d64& v);
@@ -442,7 +442,8 @@ friend class method_info;
 private:
 	SystemState* m_sys;
 	pthread_t t;
-	bool terminated;
+	enum STATUS { CREATED=0, STARTED, TERMINATED };
+	STATUS status;
 
 	llvm::Module* module;
 
@@ -624,12 +625,15 @@ public:
 		@pre shutdown must have been called
 	*/
 	~ABCVm();
+	/**
+	  	Start the VM thread
+	*/
+	void start();
 	static void Run(ABCVm* th);
 	static ASObject* executeFunction(SyntheticFunction* function, call_context* context);
 	bool addEvent(EventDispatcher*,Event*) DLL_PUBLIC;
 	int getEventQueueSize();
 	void shutdown();
-	void wait();
 
 	void pushObjAndLevel(ASObject* o, int l);
 	thisAndLevel popObjAndLevel();
@@ -639,13 +643,14 @@ public:
 	}
 
 	static bool strictEqualImpl(ASObject*, ASObject*);
+	static void publicHandleEvent(EventDispatcher* dispatcher, Event* event);
 
 };
 
 class DoABCTag: public ControlTag
 {
 private:
-	UI32 Flags;
+	UI32_SWF Flags;
 	STRING Name;
 	ABCContext* context;
 	pthread_t thread;
@@ -658,8 +663,8 @@ public:
 class SymbolClassTag: public ControlTag
 {
 private:
-	UI16 NumSymbols;
-	std::vector<UI16> Tags;
+	UI16_SWF NumSymbols;
+	std::vector<UI16_SWF> Tags;
 	std::vector<STRING> Names;
 public:
 	SymbolClassTag(RECORDHEADER h, std::istream& in);
